 /*****************************************************************************
 *                                                                            *
 *   Copyright (C) 2004-2006 by Michael Schulze                               *
 *   mike.s@genion.de                                                         *
 *                                                                            *
 *  The code contained in this file is free software; you can redistribute    *
 *  it and/or modify it under the terms of the GNU Lesser General Public      *
 *  License as published by the Free Software Foundation; either version      *
 *  2.1 of the License, or (at your option) any later version.                *
 *                                                                            *
 *  This file is distributed in the hope that it will be useful,              *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 *  Lesser General Public License for more details.                           *
 *                                                                            *
 *  You should have received a copy of the GNU Lesser General Public          *
 *  License along with this code; if not, write to the Free Software          *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *  iTunes and iPod are trademarks of Apple                                   *
 *                                                                            *
 *  This product is not supported/written/published by Apple!                 *
 *****************************************************************************/

#ifndef IPOD_H
#define IPOD_H

#include <qfile.h>
#include <qstring.h>

#include "itunesdb.h"

#include "ipoddevicedetails.h"
#include "ipodsysinfo.h"

#define LOGFILEPREFIX "/kio_ipod-"
#define LOG_DEFAULT true

/**
 * @mainpage libqtpod - a C++ library providing access to the contents of an Apple iPod
 *
 * This library provides access to various information stored on an Apple iPod.
 * @n
 * At the moment this still depends on Trolltechs Qt library and I'll need help changing that
 * for future versions since as a library we should only depend on the standard C++ libraries.
 * @section Usage
 * The following chapter gives a short introduction on how to use the library to open the device
 * and access @ref itunesdb "tracks, playlist" and @ref statistics "disc usage statistics".
 * @n
 * @section ipod The IPod class
 * To access the contents of the ipod an instance of the IPod class needs to be instantiated
 * with the mountpoint/drive letter of the device and opened with a call to the IPod::open() method.
 * @n For linux this would probably something like that
 * @code
 * IPod ipod( "/media/ipod" );
 * @endcode
 * and
 * @code
 * IPod ipod( "E:" );
 * @endcode
 * in a Windows environment.
 * @see IPodMountPoint::mountedIPods() returning a list of ipods connected to the system.
 * The IPodMountPoint::getMountPoint() method returns the String to initialize the IPod class with.
 *   
 * Open the device with open()
 * @code
 * if( !ipod.open() ) {
 *     // failed
 * }
 * @endcode
 * After the IPod instance is opened the information stored on the device is accessible and can be retrieved.
 * @n
 * @section itunesdb Tracks and Playlists
 * Information about the tracks and playlists on the iPod device is stored in a database called ITunesDB and
 * libqtpod contains the class ITunesDB representing this database.
 * @n The now opened IPod instance holds an instance of this class which is accessible thru the IPod::getITunesDB() method:
 * @code
 * // get the iTunesDB instance
 * ITunesDB& itunesdb = ipod.getITunesDB();
 * @endcode
 * @n Now we can iterate over the tracks stored on the device and print out artist, album, title and filename like this
 * @code
 * // get an Iterator over all tracks found on the device 
 * ITunesDB::TrackConstIterator iter = itunesdb.getAllTracks();
 * while( iter.hasNext() ) {
 *     // get the next track
 *     ITunesDBTrack * track = iter.next();
 *     // print track meta data
 *     printf( "%s\t%s\t%s\t%s\n",
 *         track->getArtist().ascii(),
 *         track->getAlbum().ascii(),
 *         track->getTitle().ascii(),
 *         itunesdb.getFileForPathInfo( track->getPathInfo() ).ascii() );
 * }
 * @endcode
 * @see ListTracksTest::run() in @ref listtrackstest.cpp "listtrackstest.cpp" in the "tests" directory for another example on how to list the tracks sorted by artist
 * 
 * Changes made to the database can be easily written back to the device by calling the IPod::synchronize() method:
 * @code
 * // synchronize with the device
 * ipod.synchronize();
 * @endcode
 * @n finally close your IPod instance when done
 * @code
 * ipod.close();
 * @endcode
 * @see @ref listtests.cpp "listtests.cpp" showing listing of tracks, playlist and creating smart playlists
 * @see Other Testcases in the classes implementing Test in the "tests/" directory 
 * @n
 * @section statistics Device Statistics
 * Access to the device statistics is implemented by the class IPodSysInfo accessible by calling getSysInfo() on your opened
 * IPod instance:
 * @code
 * IPodSysInfo& sysinfo = ipod.getSysInfo();
 * @endcode
 */

/**
 * @example listtrackstest.cpp
 * Example 1: listing of tests/listtrackstest.cpp
 */

/**
 * @example listtests.cpp
 * Example2 - listtests.cpp showing listing of tracks, playlist and creating smart playlists
 */

/**
This class represents all functionalities to access and modify information about the iPod/itunesDB

@author Michael Schulze
*/
class IPod {
public:

    /**
     * @brief Constructs a new IPod instance for an iPod at the given path.
     * Does not actually read the contents - you need to open it with @c open().
     */
    IPod( const QString& ipodBase );

    /**
     * @brief The destructor.
     */
    virtual ~IPod();

    /**
     * @brief Tries to open an ipod.
     * 
     * That means reading all important information from the device.
     * @return true if successful, otherwise false
     */
    bool open();

    /**
     * @brief Initializes this ipod by creating the nessessary directories and files.
     * 
     * This IPod instance then contains the information about the
     * initialized ipod - no need to call open() afterwards.
     */
    void initialize( const QString& title );

    /**
     * @brief Returns true if open() was called successfully for this instance.
     * @return true if open() was called successfully for this instance.
     */
    bool isOpen();

    /**
     * @brief Returns true if the ipod represented by this instance is still connected to the system
     * @return true if the ipod represented by this instance is still connected to the system false otherwise
     */
    bool isStillConnected();

    /**
     * @brief Clears all data we got from this instance whether or not it was changed.
     * 
     * Resets the state of this instance.
     */
    void close();

    /**
     * @brief Returns the name of the iPod
     * @return the name of the iPod as set in the device details
     */
    QString getName() const;

    /**
     * @brief Sets the name of the iPod.
     * 
     * When the iPod is not opened (isOpen() == false) nothing is changed
     * @param name the new name for the iPod
     */
    void setName(const QString& name);

    /**
     * @brief Returns the error occured during read or write
     * @return the error occured during read or write
     */
    const QString& getItunesDBError() const;

    /**
     * @brief Returns the IPodSysInfo for this IPod instance (const).
     * @return the IPodSysInfo for this IPod instance.
     */
    const IPodSysInfo& getSysInfo() const;

    /**
     * @brief Returns the IPodSysInfo for this IPod instance.
     * @return the IPodSysInfo for this IPod instance.
     */
    IPodSysInfo& getSysInfo();

    /**
     * @brief Returns the ITunesDB for this iPod (const).
     * @return the ITunesDB for this iPod.
     */
    const ITunesDB& getITunesDB() const;

    /**
     * @brief Returns the ITunesDB for this iPod.
     * @return the ITunesDB for this iPod.
     */
    ITunesDB& getITunesDB();

    /**
     * @brief Writes back all the changed data to the device.
     * 
     * This only includes control structures like the iTunesDB and DeviceInfo data.
     */
    void synchronize();

    /**
     * @brief Returns true if this instances data differ somehow from the original
     * 
     * data from the iPod and need to be synchronized.
     */
    bool isDirty() const { return m_itunesdb.isDirty(); }

    /**
     * @brief Lock control functions to prevent concurrent access to the iPod from different programs
     * 
     * locks the iPod
     * @param write_lock if true the ipod will be locked exclusively
     * @attention not implemented for win32
     */
    void lock(bool write_lock);

    /**
     * @brief Returns true if the iPod got locked
     * @return true if the iPod got locked
     */
    bool isLocked();

    /**
     * @brief unlock an iPod previously locked
     */
    void unlock();

    /**
     * @brief Returns the base path of the iPod
     * 
     * (i.e. "/media/ipod" on linux or "E:" on a windows machine)
     * @return the base path of the iPod
     */
    const QString& getBasePath() const { return m_ipodBase; }

    /**
     * This method creates a string containing the ipod base path encoded
     * being distinct between different iPods simultanously connected to the system.
     * Most users wont need this method - it's here as a relict from the old ipodslave code
     * and may go away in the future
     * @deprecated this method doesn't belong here
     */
    static QString createDistinctIPodName(const IPod& ipod) __attribute__((deprecated));

    // This method can be removed when we're able to handle podcasts
    bool hasPodcasts() { return m_itunesdb.hasPodcasts(); }

    /**
     * @brief Finds all the files in the Music folders that don't have a track record
     * 
     * The fully qualified pathes will be added to the given list.
     * @param list the list to add the unreferences files to
     * @return the reference to the given list
     */
    QStringList& findUnreferencedMusicFiles( QStringList& list ) const;

protected:
    QString m_ipodBase;

private:
    IPodSysInfo m_sysInfo;
    ITunesDB m_itunesdb;
    IPodDeviceDetails m_deviceDetails;

};

#endif
