 /*****************************************************************************
 *   Copyright (C) 2004 by Michael Schulze                                    *
 *   mike.s@genion.de                                                         *
 *                                                                            *
 *  The code contained in this file is free software; you can redistribute    *
 *  it and/or modify it under the terms of the GNU Lesser General Public      *
 *  License as published by the Free Software Foundation; either version      *
 *  2.1 of the License, or (at your option) any later version.                *
 *                                                                            *
 *  This file is distributed in the hope that it will be useful,              *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 *  Lesser General Public License for more details.                           *
 *                                                                            *
 *  You should have received a copy of the GNU Lesser General Public          *
 *  License along with this code; if not, write to the Free Software          *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *  iTunes and iPod are trademarks of Apple                                   *
 *                                                                            *
 *  This product is not supported/written/published by Apple!                 *
 *****************************************************************************/

#ifndef ITUNESDBLISTITEM_H
#define ITUNESDBLISTITEM_H

#include <qglobal.h>
#include <qpair.h>
#include <qstring.h>

#include <map>
#include <vector>

namespace itunesdb {

/**
 * MHOD Property types. 
 */
enum ItemProperty {
  MHOD_TITLE = 1,
  MHOD_PATH = 2,
  MHOD_ALBUM = 3,
  MHOD_ARTIST = 4,
  MHOD_GENRE = 5,
  MHOD_FDESC = 6,
  MHOD_EQ_SETTING = 7,
  MHOD_COMMENT = 8,
  MHOD_CATEGORY = 9,
  MHOD_COMPOSER = 12,
  MHOD_GROUPING = 13,
  MHOD_DESCRIPTION_TEXT = 14,
  MHOD_PODCAST_ENCLOSURE_URL = 15,
  MHOD_PODCAST_RSS_URL = 16,
  MHOD_CHAPTER_DATA = 17,
  MHOD_SUBTITLE = 18,
  MHOD_TVSHOW = 19,
  MHOD_TVEPISODE = 20,
  MHOD_TVNETWORK = 21,
  MHOD_ALBUMARTIST = 22,
  MHOD_ARTIST_FOR_SORT = 23,
  MHOD_KEYWORDS = 24,
  MHOD_TITLE_FOR_SORT = 27, // TODO
  MHOD_ALBUM_FOR_SORT = 28, // TODO
  MHOD_ALBUMARTIST_FOR_SORT = 29,   // TODO
  MHOD_COMPOSER_FOR_SORT = 30,  // TODO
  MHOD_SMART_PLAYLIST_INFO = 50,
  MHOD_SMART_PLAYLIST_IRULES = 51,
  MHOD_LIBRARY_PLAYLIST_INDEX = 52,
  MHOD_PLAYLIST = 100
};

/**
 * Possible item types
 */
enum {
  ITEMTYPE_NONE = 0,
  ITEMTYPE_TRACK = 1,
  ITEMTYPE_PLAYLISTITEM = 2,
  ITEMTYPE_PLAYLIST = 3
};    // known implementors

typedef QPair<Q_UINT32,QByteArray> MHODPair;

/**
Describes a list item in iTunesDB. Possible known subtypes at the moment are playlist, playlistitem and track

@author Michael Schulze
*/
class ListItem {

    friend class ItunesDBParser;

public:

    class LessThanStringPropertyComparator {
        ItemProperty mm_propertyToCompare;
    public:
        LessThanStringPropertyComparator( ItemProperty propertyToCompare )
            : mm_propertyToCompare( propertyToCompare ) {}
        bool operator() ( ListItem* item1, ListItem* item2 ) {
            return item1->getItemProperty( mm_propertyToCompare ).localeAwareCompare( item2->getItemProperty( mm_propertyToCompare ) ) < 0;
        }
    };

    virtual ~ListItem();

    /**
     * returns the type ofthis item
     * possible values from the known implementors are ITEMTYPE_PLAYLIST, ITEMTYPE_PLAYLISTITEM
     * and ITEMTYPE_TRACK
     */
    int getType() const;

    /**
     * sets a given itunesDB item property
     * @param data value for the property to be set
     * @param field propertyID of the property to be set
     */
    void setItemProperty(const QString& data, ItemProperty field);

    /**
     * Returns true if the given field is set with a non empty value.
     * @param field PropertyID of the property to be checked.
     * @return true if the given field is set with a non empty value.
     */
    bool containsItemProperty( ItemProperty field ) const;

    /**
     * Returns the value for the given property.
     * @param field PropertyID of the property to be returned.
     * @return the value for the given property.
     */
    const QString& getItemProperty( ItemProperty field) const;

    /**
     * Returns the number of properties.
     */
    uint getNumComponents() const;

    /**
     * Returns the number of String properties.
     */
    uint getNumStringComponents() const;

    /**
     * This method may be overridden for consistency checks after all properties have been read.
     */
    virtual void doneAddingData();

    /**
     * Returns true if this database item is out of sync with the database. This is true
     * if/when the item got changed and did not get saved back to the database yet.
     */
    virtual bool isDirty() const;

protected:
    typedef std::map<Q_UINT32,QString> PropertyMap;

    PropertyMap m_properties;

    ListItem();
    ListItem( int type);

    const QByteArray * getDumpedMHOD( Q_UINT32 type ) const;

    virtual void readMHOD( QDataStream& stream, Q_UINT32 type, Q_UINT32 blocklen );
    Q_UINT32 writeMHODsToStream ( QDataStream & outstream ) const;

    virtual void readNonStringMHOD( QDataStream& stream, Q_UINT32 type, Q_UINT32 blocklen );
    virtual uint writeNonStringMHODs( QDataStream& outstream ) const;

    typedef std::vector<MHODPair> DumpedMHODs;
    DumpedMHODs m_dumpedMHODs;

    int m_itemType;

    virtual void setDirty( bool dirtyFlag = true );

private:
    bool m_dirty;
};

}

#endif
